#[non_exhaustive]pub enum GridError {
OutOfBounds {
pos: Pos,
},
}Expand description
An error type for operations on or creating a Grid.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
OutOfBounds
Could not access an element at a specified position due to it being out of bounds.